package heap;

// The heapsort algorithm - class NodeArray.

// This implementation is modeled after the courseware Heapsort-SALA.
// http://OLLI.Informatik.Uni-Oldenburg.DE/heapsort_SALA/
// It is optimized for easy understanding and not for efficiency.
// (c) 2001 Nils Faltin. Distributed under the GNU General Public License. 
// The license text ist contained in the courseware distribution.

// Base class of HeapArray
// Contains an array of nodes that can be compared and swapped. Thus provides the type of array used by classical sorting algorithms like heapsort.
// provides 
// - specialized access methods to nodes
// - compare nodes
// - swap nodes
// - add a Node
// - String of node strings

abstract public class NodeArray {
	private Node [] array; // array holds heap tree followed by the result list. 
	private int array_last = -1; // index of last node in array

	static void output(Object s){
	  System.out.println(s.toString());
	}

   // constructor: initialize with empty array of length len
   NodeArray(int len){
     array = new Node[len];
   }
   
   NodeArray(){
     this(0);
   }
   
   // instance methods
   
   // debugging support
   
   public String toString(){
     StringBuffer s = new StringBuffer();
     for (int i=0; i < array.length; i++){
       s.append( array[i] + " ");
     }
     return "(" + s + ")";
   }
   
   // services
   
   // access to Nodes
   
   // add a node after the last node
   public void addNode(Node n){
	 array[++array_last] = n;
   }
   
   // read out Array
   public Node getNodeAt(int k){
	 return array[k];
   }
   
   // register that last Node has been removed
   protected void decrementArraysize(){
	 array_last--;
   }

   // is Node at index a greater than Node at index b?
   protected boolean isGreaterThan(int a, int b){
     return array[a].isGreaterThan(array[b]);
   }
   
   // Swap the nodes at the indices a and b in the array.
   protected void swap(int a, int b){
      Node t = array[a];
      array[a] = array[b];
      array[b] = t;
   }
   



}
